#ifndef PPANELS_H
#define PPANELS_H

#include "sdpm.h"
#include <vector>
#include <cmath>
#include <Eigen/Dense>

namespace sdpm
{

class SDPM_API Panels
{

private:

public:
    double c0 = 2; /* Root chord (m) */
    double _lambda = 0.3; /* Taper ratio (-) */
    double epsilon = 3*M_PI/180; /* Twist angle (rad) */
    double LambdaC4 = 35*M_PI/180; /* Sweep angle at 1/4 chord (rad) */
    double AR = 7;
    double b; /* Span (m) */
    double dihedral = 3*M_PI/180; /* Dihedral angle (rad) */
    double ax = 1/2; /* Axis around which twist and taper are defined */

    double maxcamb = 0.02;
    double maxpos = 0.4;
    double thick = .12; // Airfoil maximum thickness


    double beta;
    double Surface;

    size_t nPanChord, nPanSpan;

    /* Bound panel corner point matrices */
    Eigen::MatrixXd Xp; // Size (2m+1, n)
    Eigen::MatrixXd Yp;
    Eigen::MatrixXd Zp;

    /* Compressibility scaling */
    Eigen::MatrixXd Xp0; // =Xp;
    Eigen::MatrixXd Yp0; // =Yp;
    Eigen::MatrixXd Zp0; // =Zp;

    /* Control points */
    Eigen::MatrixXd Xc; // =zeros(2*m,n);
    Eigen::MatrixXd Yc;
    Eigen::MatrixXd Zc;
    Eigen::MatrixXd nx;
    Eigen::MatrixXd ny;
    Eigen::MatrixXd nz;
    Eigen::MatrixXd tauxx;
    Eigen::MatrixXd tauxy;
    Eigen::MatrixXd tauxz;
    Eigen::MatrixXd tauyx;
    Eigen::MatrixXd tauyy;
    Eigen::MatrixXd tauyz;

    Eigen::MatrixXd Xc0;
    Eigen::MatrixXd Yc0;
    Eigen::MatrixXd Zc0;

    Eigen::MatrixXd s0;
    Eigen::MatrixXd cpln;

    /* Surface */
    Eigen::MatrixXd s;

    /* Wake */
    Eigen::MatrixXd Xw;
    Eigen::MatrixXd Yw;
    Eigen::MatrixXd Zw;

    /* Wake */
    Eigen::MatrixXd MAw; // (mw, n)
    Eigen::MatrixXd Xcw;
    Eigen::MatrixXd Ycw;
    Eigen::MatrixXd Zcw;
    Eigen::MatrixXd nxw;
    Eigen::MatrixXd nyw;
    Eigen::MatrixXd nzw;
    Eigen::MatrixXd tauxxw;
    Eigen::MatrixXd tauxyw;
    Eigen::MatrixXd tauxzw;
    Eigen::MatrixXd tauyxw;
    Eigen::MatrixXd tauyyw;
    Eigen::MatrixXd tauyzw;
    Eigen::MatrixXd cp; // (2mw, n)

    /* Normal vectors of compressible geometry panels */
    Eigen::MatrixXd nx0;
    Eigen::MatrixXd ny0;
    Eigen::MatrixXd nz0;

    /* Influence coefficients */

    /* Sources */
    Eigen::MatrixXd AN;
    Eigen::MatrixXd Au;
    Eigen::MatrixXd Av;
    Eigen::MatrixXd Aw;
    /* Doublets */
    Eigen::MatrixXd BN;
    Eigen::MatrixXd Bu;
    Eigen::MatrixXd Bv;
    Eigen::MatrixXd Bw;

    /* Wake Doublets only */
    Eigen::MatrixXd CN;
    Eigen::MatrixXd Cu;
    Eigen::MatrixXd Cv;
    Eigen::MatrixXd Cw;

    Eigen::MatrixXd Kutta;

    Panels(size_t nPanChord);

    //void CreateWake(double U, double V, double W, double dt);
    void ComputeInfluenceCoeffs_wing();
    void ComputeInfluenceCoeffs_wake(unsigned long it);

    void sdpanel_wing(Eigen::RowVector2d &Phisd, Eigen::RowVectorXd &uvw, double x0, double y0, double z0, Eigen::RowVectorXd x, Eigen::RowVectorXd y, Eigen::RowVectorXd z, double xcp, double ycp, double zcp,
        Eigen::RowVector3d normal, Eigen::RowVector3d taux, Eigen::RowVector3d tauy);
    
    void sdpanel_wake(Eigen::RowVectorXd &Phisd, Eigen::RowVectorXd &uvw,double x0, double y0, double z0, Eigen::RowVectorXd x, Eigen::RowVectorXd y, Eigen::RowVectorXd z, double xcp, double ycp, double zcp,
        Eigen::RowVector3d normal, Eigen::RowVector3d taux, Eigen::RowVector3d tauy);
    
    void WakeControlPoints(unsigned long it);


    void SetKutta();
    

    double Getc0(){return c0;};

    void CreateWing();

};
} // namespace dart
#endif //PPANELS_H
