#ifndef WVISCMESH_H
#define WVISCMESH_H

#include "vii.h"
#include "wBLRegion.h"

namespace vii
{

class VII_API ViscMesh
{
  private:
    std::vector<double> x,
                        y,
                        z,
                        Loc,    /* BL (local) coordinates */
                        Ext,    /* Coordinates at the edge of the BL */
                        dx,
                        alpha;  /* Angle of the element for the rotation matrix */

    unsigned int nMarkers;
    unsigned int nElm;


  public:
    unsigned int prevnMarkers;
    
    ViscMesh();
    ~ViscMesh();
    
    unsigned int GetnMarkers() const {return nMarkers;};
    double GetLoc(size_t iPoint) const {return Loc[iPoint];};
    double Getx(size_t iPoint) const {return x[iPoint];};
    double Gety(size_t iPoint) const {return y[iPoint];};
    double Getz(size_t iPoint) const {return z[iPoint];};
    double GetExt(size_t iPoint) const {return Ext[iPoint];};
    double GetAlpha(size_t iElm) const {return alpha[iElm];};

    int GetDiffnElm() const {return nMarkers-prevnMarkers;};

    void SetGlob(std::vector<double> _x, std::vector<double> _y, std::vector<double> _z);
    void SetExt(std::vector<double> ExtM) {Ext.resize(ExtM.size(), 0.); Ext = ExtM;};
    
    void ComputeBLcoord();

};
} // namespace dart
#endif //WVISCMESH_H